/** *****************************************************************************************
 * @class: MouseMoveTracker
 * @classDescription: implements an an tracking of the user mouse moves to get relevant screen sections
 * @author: Sebastian Martens
 * @SVN: $Id$
 ** *****************************************************************************************/
dojo.declare("MouseMoveTracker", null, {
	
	version: "$Revision: 5 $",
	
	interval: 500, // number of milliseconds mouse corrdinate should be saved
	
	idleStatusNumber: 5, // maximum values to save of one point if mouse isn't moved
	idleCount:0, // current counter of saved values of same coordinate
	rootNodeId: "",
	rootNode:null,
	
	actionURI: "",
	actionURIOverlay: "",
	
	location: "",
	caughtPoints: [],
	coords: null,
	
	mouseX:-1,
	mouseY:-1,
	savedX:-1,
	savedY:-1,
	
	
	/**
	 * @constructor
	 * @param {Object} obj - construction parameter
	 */
	constructor: function( obj ){
		dojo.mixin(this,obj);
		
		if( this.actionURI.length<1 ) return false;
		
		// body is default root node
		this.rootNode = (this.rootNodeId.length)?dojo.byId(this.rootNodeId):dojo.query("BODY")[0];
		
		if( this.rootNode ){
			this.coords = dojo.coords( this.rootNode );
		}else return false;
		
		this.location = window.location.href;
		
		// show overlay mode
		if( window.location.hash == "#overlay" ){
			this.buildOverlay();
			
		// standard mode data collection
		}else{
			// set reference
			document['mouseMoveTrackerInstance'] = this;
			
			
			dojo.connect(document,"onmousemove",this,this.mouseMoveHandler);
			
			// onload handler to submit selected data
			dojo.addOnUnload(function(){
				document['mouseMoveTrackerInstance'].collectSubmit();
			});
			
			// start timed handler
			this.timedHandler();
		}
		
	},
	
	/**
	 * does request to 
	 */
	buildOverlay: function(){
		if(dojo.isFF) console.info("Start Overlay Mode");
		dojo.xhrPost({
			'url':this.actionURIOverlay,
			'content':{
				'getdata': this.location
			},
			'handleAs':'json',
			'scope':this,
			'handle': this._dataHandler,
			'preventCache':true
		});
	},
	
	/**
	 * 
	 * @param obj
	 * @param ioArgs
	 * @returns
	 */
	_dataHandler: function( obj, ioArgs ){
		if( ioArgs && ioArgs.args && ioArgs.args.scope ){
			ioArgs.args.scope.createOverlay( obj );
		}
	},
	
	/**
	 * 
	 * @param obj
	 * @param val
	 * @returns
	 */
	getBackgroundByValue: function(obj, val){
		if( val!=0){
			thisVal = Math.floor( (10/obj.max) * val );
			switch( thisVal ){
				case 1: return "#02bf1d";
				case 2: return "#68e701";
				case 3: return "#bee701";
				case 4: return "#aeff00";
				case 5: return "#fff600";
				case 6: return "#ffd200";
				case 7: return "#ffc000";
				case 8: return "#ff9c00";
				case 8: return "#ff7200";
				case 10: return "#ff0000";
			}
		}
		return "transparent";
	},
	
	/**
	 * 
	 * @param obj
	 * @returns
	 */
	createOverlay: function( obj ){
		obj = obj[0];
		height = obj.max_y * obj.gridsize;
		width = obj.max_x * obj.gridsize;
		
		dojo.create("DIV",{'style':'z-index:5000000;position:absolute;top:'+this.coords.y+'px;left:'+this.coords.x+'px;width:'+width+'px;height:'+height+'px','id':'mouseMoveTrackerOverlay'},dojo.body());
		
		var node = dojo.byId('mouseMoveTrackerOverlay');
		for(var y=1;y<=obj.max_y;y++){
			for(var x=1;x<=obj.max_x;x++){
				val = (obj.data[x] && obj.data[x][y])?obj.data[x][y]:0;
				dojo.create("DIV",{'count':val,'style':'float:left;width:'+obj.gridsize+'px;height:'+obj.gridsize+'px;opacity:0.6;-ms-filter:"progid:DXImageTransform.Microsoft.Alpha(Opacity=60)";filter: alpha(opacity=60);background-color:'+this.getBackgroundByValue(obj,val)+';'},node );
			}
		}
	},
	
	/**
	 * eventhandler for any mouse move event on the page. saves current mouse position to 
	 * local vars. timed event will store these values to array
	 * @param Event evt - mouse move event object
	 */
	mouseMoveHandler: function( evt ) {
		if ( dojo.isIE ) {
			this.mouseX = event.clientX + document.body.scrollLeft;
			this.mouseY = event.clientY + document.body.scrollTop; 
		}else{
			this.mouseX = evt.pageX;
			this.mouseY = evt.pageY;
		}
	},
	
	/**
	 * adds the current point to local array
	 * @returns
	 */
	addPoint: function(){
		this.savedX = this.mouseX;
		this.savedY = this.mouseY;
		
		this.caughtPoints.push({
			'x':(this.mouseX-this.coords.x),
			'y':(this.mouseY-this.coords.y)
		});
	},
	
	/**
	 * saves current mouse coordinates to loacal array.
	 */
	timedHandler: function(){
		if( this.savedX == this.mouseX && this.savedY == this.mouseY ){
			this.idleCount++;
			if( this.idleCount<this.idleStatusNumber ){
				this.addPoint();
			}
		}else{
			this.addPoint();
			this.idleCount = 0;
		}
		
		window.setTimeout("document['mouseMoveTrackerInstance'].timedHandler()",this.interval);
	},
	
	/**
	 * collects the saved data and submits it to server
	 */
	collectSubmit: function(){
		// save current mouse position
		this.timedHandler();
		
		dojo.xhrPost({
			'url':this.actionURI,
			'content':{
				'data':dojo.toJson({ 
					'location':this.location,
					'data':this.caughtPoints 
					
				})
			},
			'preventCache':true,
			'sync':true
		});
	}
	
});
/** END MouseMoveTracker */